/*
 * ARJ+xenu is based on open-source ARJ by Andrew Belov <andrew_belov@newmail.ru>.
 *
 * Copyright (C) 2011 Tomasz Konojacki <xenu@poczta.onet.pl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * ---------------------------------------------------------------------------
 * Handy functions for dealing with extended headers.
 *
 */

#include "arj.h"

DEBUGHDR(__FILE__)                      /* Debug information block */

/* Allocates a per-file extended header structure */

struct ext_hdr FAR *eh_alloc()
{
 struct ext_hdr FAR *rc;

 rc=(struct ext_hdr FAR *)farmalloc_msg(sizeof(struct ext_hdr));
 far_memset((char FAR *)rc, 0, sizeof(struct ext_hdr));
 return(rc);
}

/* Locates a block with the specified tag, returning pointer to it */

struct ext_hdr FAR *eh_lookup(struct ext_hdr FAR *eh, char tag)
{
 while(eh->next!=NULL)
 {
  if(eh->tag==tag)
   return(eh);
  eh=eh->next;
 }
 return(NULL);
}

/* Locates an unfinalized block */

struct ext_hdr FAR *eh_find_pending(struct ext_hdr FAR *eh)
{
 if(eh==NULL)
  return(NULL);
 while(eh->next!=NULL)
 {
  if(EH_STATUS(eh)!=EH_FINALIZED)
   return(eh);
  eh=eh->next;
 }
 return(NULL);
}

/* Inserts a new block into an instantiated extended header structure. If the
   block is given as NULL, performs reallocation only */

struct ext_hdr FAR *eh_append(struct ext_hdr FAR *eh, char tag, char FAR *block, unsigned int size)
{
 struct ext_hdr FAR *p_eh;

 if((p_eh=eh_lookup(eh, tag))==NULL)
 {
  for(p_eh=eh; p_eh->next!=NULL; p_eh=p_eh->next);
  p_eh->tag=tag;
  p_eh->next=eh_alloc();
 }
 p_eh->raw=(char FAR *)farrealloc_msg(p_eh->raw, p_eh->size+size);
 if(block!=NULL)
  far_memmove(p_eh->raw+p_eh->size, block, size);
 p_eh->size+=size;
 return(p_eh);
}

/* Releases the extended header structure */

void eh_release(struct ext_hdr FAR *eh)
{
 struct ext_hdr FAR *p_eh;

 while((p_eh=eh->next)!=NULL)
 {
  if(eh->raw!=NULL)
   farfree(eh->raw);
  farfree(eh);
  eh=p_eh;
 }
}
